import urllib.request
import json
import math
import sched, time

class TJBot:
    def __init__(self):
        # Initializer

        self.updates_ = {}
        self.updateId_ = 0
        self.scheduler_ = sched.scheduler(time.time, time.sleep)
        self.subscribedChats_ = []
        self.delayTime_ = 2
        self.serviceLength_ = 347
        self.kiisuUpdated_ = False
        self.kiisuLink_ = ""

        self.kiisuCoolDown_ = 0
        self.telegramCoolDown_ = 0

        # Read options for bot
        self.readOptionFile()

    def readOptionFile(self):
        # Reads an option file and sets variables accordingly

        file = open("TJOptions.txt", "r")
        if file.closed:
            print("Couldn't read options")
            self.updateId_ = 0;
            return 1

        data = file.read()
        j_data = json.loads(data)

        # Read the latest update-Id
        self.updateId_ = j_data['update_id']
        self.subscribedChats_ = j_data["subscribedChats"]
        self.kiisuUpdated_ = j_data["kiisuUpdated"]
        self.kiisuLink_ = j_data["kiisuLink"]
        self.delayTime_ = j_data["delayTime"]
        self.serviceLength_ = j_data["serviceLength"]

        file.close()
        return 0

    def writeOptionFile(self):
        # Writes current status to an option file

        data = json.dumps({"delayTime": self.delayTime_, "serviceLength": self.serviceLength_,"update_id": self.updateId_, "subscribedChats": self.subscribedChats_, "kiisuUpdated": self.kiisuUpdated_, "kiisuLink": self.kiisuLink_})

        file = open("TJOptions.txt", "w")
        if file.closed:
            print("Couldn't write options")
            return 1

        file.write(data)
        file.close()
        return 0

    def handleUpdates(self):
        # Handles any and all new updates

        # Remind TJ at 0530 AM
        currentHour = time.gmtime(time.time()).tm_hour

        if currentHour > 3 and not self.kiisuUpdated_:
            self.updateKiisu()
            self.kiisuUpdated_ = True
        if currentHour < 3 and self.kiisuUpdated_:
            # New day
            self.kiisuUpdated_ = False

        for subsChat in self.subscribedChats_:
            if currentHour > 5 and not subsChat["remindedToday"]:
                self.getTJ(subsChat["id"])
                subsChat["remindedToday"] = True

            if currentHour < 5 and subsChat["remindedToday"]:
                # We have apparently arrived on a new day, reset reminder
                subsChat["remindedToday"] = False

        # Check for any commands given
        for update in self.updates_:
            if update["update_id"] > self.updateId_-1:
                self.updateId_ = update["update_id"]+1
                print("Received update:\n") # DEBUG !!
                print(str(update).encode("utf-8",errors="ignore"))  # DEBUG !!
                print("------------------") # DEBUG !!
                # Write new status to option file
                self.writeOptionFile()

            # For non-text-message updates, we stop here
            if not "message" in update:
                continue
            if not "text" in update["message"]:
                continue

            if "/getTJ" in update["message"]["text"]:
                self.getTJ(update["message"]["chat"]["id"])

            if "/start" in update["message"]["text"]:
                newChatId = update["message"]["chat"]["id"]
                chatAlreadyListed = False

                for chat in self.subscribedChats_:
                    if chat["id"] == newChatId:
                        chatAlreadyListed = True

                if not chatAlreadyListed:
                    # The chat is subscribing morning-calls
                    self.subscribedChats_.append({"id": newChatId, "remindedToday": False})
                    # Write new status to option file
                    self.writeOptionFile()


    def getUpdates(self):
        # Get updates from the API

        # Skip if cooldown
        if (self.telegramCoolDown_ > 0):
            self.telegramCoolDown_ -= 1
            self.scheduler_.enter(self.delayTime_, 10, self.getUpdates)
            return 1

        try:
				# Valid BOTID required here
                request = urllib.request.urlopen("https://api.telegram.org/BOTID/getUpdates?offset="+str(self.updateId_)+"")
                data = request.read().decode('utf-8')
                j_data = json.loads(data)
                result = j_data["result"]

                request.close()

                self.updates_ = result;
                self.scheduler_.enter(self.delayTime_, 10, self.getUpdates)
                self.handleUpdates()
        except:
                print("An error occurred with TG-API")
                self.telegramCoolDown_ = 300/self.delayTime_
                self.scheduler_.enter(self.delayTime_, 10, self.getUpdates)
                return 1

        return 0

    def sendMessage(self, toId, text):
        # Send a message to chat id toId with text as content

        try:
				# Valid BOTID required here
                request = urllib.request.urlopen("https://api.telegram.org/BOTID/sendMessage?chat_id="+str(toId)+"&text="+text+"")
                request.close()
        except:
                print("An error occurred with Telegram API")
                self.telegramCoolDown_ = 300/self.delayTime_
                return 1
        return 0


    def updateKiisu(self):
        # Update todays TJ kiisu

        # Skip if on cooldown
        if (self.kiisuCoolDown_ > 0):
            self.kiisuCoolDown_ -= 1
            return 1

        try:
            request = urllib.request.urlopen("https://gelbooru.com/index.php?page=dapi&s=post&q=index&tags=cat_ears%20rating:safe&limit=1&json=1")
            data = request.read().decode("utf-8")
            j_data = json.loads(data)

            self.kiisuLink_ = j_data[0]["file_url"]

            request.close()
        except:
            print("An error occurred with retreiving kiisu.")
            self.kiisuCoolDown_ = 300/self.delayTime_
            return 1

        return 0

    def getTJ(self, toId):
        today = math.floor(time.time()/86400)
        begin = math.floor(time.mktime(time.strptime("8/1/2018", "%d/%m/%Y"))/86400)
        passed = today - begin
        TJ = self.serviceLength_ - passed

        self.sendMessage(toId, "Komppaniassa heratys!")
        self.sendMessage(toId, "Paivan kiisu: "+self.kiisuLink_+" :3")
        self.sendMessage(toId, "TJ "+str(TJ)+"")

        return 0

    def launch(self):
        # Launch the bot

        self.scheduler_.enter(self.delayTime_, 10, self.getUpdates)
        self.scheduler_.run()
        return 0
